@@ -3,7 +3,7 @@
 // This source code is licensed under the MIT license found in the
 // LICENSE file in the root directory of this source tree.
 
-//! An implementation of a 62-bit STARK-friendly prime field with modulus 2^62 - 111 * 2^39 + 1.
+//! An implementation of a 62-bit STARK-friendly prime field with modulus $2^{62} - 111 \cdot 2^{39} + 1$.
 //!
 //! All operations in this field are implemented using Montgomery arithmetic. It supports very
 //! fast modular arithmetic including branchless multiplication and addition. Base elements are
@@ -11,7 +11,7 @@
 
 use super::{
     traits::{FieldElement, StarkField},
-    QuadExtensionA,
+    ExtensibleField,
 };
 use core::{
     convert::{TryFrom, TryInto},
@@ -157,8 +157,6 @@ impl FieldElement for BaseElement {
 }
 
 impl StarkField for BaseElement {
-    type QuadExtension = QuadExtensionA<Self>;
-
     /// sage: MODULUS = 2^62 - 111 * 2^39 + 1 \
     /// sage: GF(MODULUS).is_prime_field() \
     /// True \
@@ -184,6 +182,7 @@ impl StarkField for BaseElement {
         Self::MODULUS.to_le_bytes().to_vec()
     }
 
+    #[inline]
     fn as_int(&self) -> Self::PositiveInteger {
         // convert from Montgomery representation by multiplying by 1
         let result = mul(self.0, 1);
@@ -210,6 +209,7 @@ impl Display for BaseElement {
 // ================================================================================================
 
 impl PartialEq for BaseElement {
+    #[inline]
     fn eq(&self, other: &Self) -> bool {
         // since either of the elements can be in [0, 2M) range, we normalize them first to be
         // in [0, M) range and then compare them.
@@ -286,12 +286,79 @@ impl Neg for BaseElement {
     }
 }
 
+// QUADRATIC EXTENSION
+// ================================================================================================
+
+/// Defines a quadratic extension of the base field over an irreducible polynomial x<sup>2</sup> -
+/// x - 1. Thus, an extension element is defined as α + β * φ, where φ is a root of this polynomial,
+/// and α and β are base field elements.
+impl ExtensibleField<2> for BaseElement {
+    #[inline(always)]
+    fn mul(a: [Self; 2], b: [Self; 2]) -> [Self; 2] {
+        let z = a[0] * b[0];
+        [z + a[1] * b[1], (a[0] + a[1]) * (b[0] + b[1]) - z]
+    }
+
+    #[inline(always)]
+    fn frobenius(x: [Self; 2]) -> [Self; 2] {
+        [x[0] + x[1], -x[1]]
+    }
+}
+
+// CUBIC EXTENSION
+// ================================================================================================
+
+/// Defines a cubic extension of the base field over an irreducible polynomial x<sup>3</sup> +
+/// 2x + 2. Thus, an extension element is defined as α + β * φ + γ * φ^2, where φ is a root of this
+/// polynomial, and α, β and γ are base field elements.
+impl ExtensibleField<3> for BaseElement {
+    #[inline(always)]
+    fn mul(a: [Self; 3], b: [Self; 3]) -> [Self; 3] {
+        // performs multiplication in the extension field using 6 multiplications, 8 additions,
+        // and 9 subtractions in the base field. overall, a single multiplication in the extension
+        // field is roughly equal to 12 multiplications in the base field.
+        let a0b0 = a[0] * b[0];
+        let a1b1 = a[1] * b[1];
+        let a2b2 = a[2] * b[2];
+
+        let a0b0_a0b1_a1b0_a1b1 = (a[0] + a[1]) * (b[0] + b[1]);
+        let minus_a0b0_a0b2_a2b0_minus_a2b2 = (a[0] - a[2]) * (b[2] - b[0]);
+        let a1b1_minus_a1b2_minus_a2b1_a2b2 = (a[1] - a[2]) * (b[1] - b[2]);
+        let a0b0_a1b1 = a0b0 + a1b1;
+
+        let minus_2a1b2_minus_2a2b1 = (a1b1_minus_a1b2_minus_a2b1_a2b2 - a1b1 - a2b2).double();
+
+        let a0b0_minus_2a1b2_minus_2a2b1 = a0b0 + minus_2a1b2_minus_2a2b1;
+        let a0b1_a1b0_minus_2a1b2_minus_2a2b1_minus_2a2b2 =
+            a0b0_a0b1_a1b0_a1b1 + minus_2a1b2_minus_2a2b1 - a2b2.double() - a0b0_a1b1;
+        let a0b2_a1b1_a2b0_minus_2a2b2 = minus_a0b0_a0b2_a2b0_minus_a2b2 + a0b0_a1b1 - a2b2;
+        [
+            a0b0_minus_2a1b2_minus_2a2b1,
+            a0b1_a1b0_minus_2a1b2_minus_2a2b1_minus_2a2b2,
+            a0b2_a1b1_a2b0_minus_2a2b2,
+        ]
+    }
+
+    #[inline(always)]
+    fn frobenius(x: [Self; 3]) -> [Self; 3] {
+        // coefficients were computed using SageMath
+        [
+            x[0] + BaseElement::new(2061766055618274781) * x[1]
+                + BaseElement::new(786836585661389001) * x[2],
+            BaseElement::new(2868591307402993000) * x[1]
+                + BaseElement::new(3336695525575160559) * x[2],
+            BaseElement::new(2699230790596717670) * x[1]
+                + BaseElement::new(1743033688129053336) * x[2],
+        ]
+    }
+}
+
 // TYPE CONVERSIONS
 // ================================================================================================
 
 impl From<u128> for BaseElement {
-    /// Converts a 128-bit value into a filed element. If the value is greater than or equal to
-    /// the field modulus, modular reduction is silently preformed.
+    /// Converts a 128-bit value into a field element. If the value is greater than or equal to
+    /// the field modulus, modular reduction is silently performed.
     fn from(value: u128) -> Self {
         // make sure the value is < 4M^2 - 4M + 1; this is overly conservative and a single
         // subtraction of (M * 2^65) should be enough, but this needs to be proven
@@ -312,29 +379,29 @@ impl From<u128> for BaseElement {
 }
 
 impl From<u64> for BaseElement {
-    /// Converts a 64-bit value into a filed element. If the value is greater than or equal to
-    /// the field modulus, modular reduction is silently preformed.
+    /// Converts a 64-bit value into a field element. If the value is greater than or equal to
+    /// the field modulus, modular reduction is silently performed.
     fn from(value: u64) -> Self {
         BaseElement::new(value)
     }
 }
 
 impl From<u32> for BaseElement {
-    /// Converts a 32-bit value into a filed element.
+    /// Converts a 32-bit value into a field element.
     fn from(value: u32) -> Self {
         BaseElement::new(value as u64)
     }
 }
 
 impl From<u16> for BaseElement {
-    /// Converts a 16-bit value into a filed element.
+    /// Converts a 16-bit value into a field element.
     fn from(value: u16) -> Self {
         BaseElement::new(value as u64)
     }
 }
 
 impl From<u8> for BaseElement {
-    /// Converts an 8-bit value into a filed element.
+    /// Converts an 8-bit value into a field element.
     fn from(value: u8) -> Self {
         BaseElement::new(value as u64)
     }
@@ -344,7 +411,7 @@ impl From<[u8; 8]> for BaseElement {
     /// Converts the value encoded in an array of 8 bytes into a field element. The bytes are
     /// assumed to encode the element in the canonical representation in little-endian byte order.
     /// If the value is greater than or equal to the field modulus, modular reduction is silently
-    /// preformed.
+    /// performed.
     fn from(bytes: [u8; 8]) -> Self {
         let value = u64::from_le_bytes(bytes);
         BaseElement::new(value)
