@@ -25,7 +25,7 @@ use utils::{
 /// Moreover, it defines interfaces for serializing and deserializing field elements.
 ///
 /// The elements could be in a prime field or an extension of a prime field. Currently, only
-/// quadratic field extensions are supported.
+/// quadratic and cubic field extensions are supported.
 pub trait FieldElement:
     Copy
     + Clone
@@ -91,21 +91,28 @@ pub trait FieldElement:
     // --------------------------------------------------------------------------------------------
 
     /// Returns this field element added to itself.
+    #[inline]
+    #[must_use]
     fn double(self) -> Self {
         self + self
     }
 
     /// Returns this field element raised to power 2.
+    #[inline]
+    #[must_use]
     fn square(self) -> Self {
         self * self
     }
 
     /// Returns this field element raised to power 3.
+    #[inline]
+    #[must_use]
     fn cube(self) -> Self {
         self * self * self
     }
 
     /// Exponentiates this field element by `power` parameter.
+    #[must_use]
     fn exp(self, power: Self::PositiveInteger) -> Self {
         let mut r = Self::ONE;
         let mut b = self;
@@ -133,9 +140,11 @@ pub trait FieldElement:
 
     /// Returns a multiplicative inverse of this field element. If this element is ZERO, ZERO is
     /// returned.
+    #[must_use]
     fn inv(self) -> Self;
 
     /// Returns a conjugate of this field element.
+    #[must_use]
     fn conjugate(&self) -> Self;
 
     // SERIALIZATION / DESERIALIZATION
@@ -189,11 +198,8 @@ pub trait FieldElement:
 ///
 /// A STARK-friendly field is defined as a prime field with high two-addicity. That is, the
 /// the modulus of the field should be a prime number of the form `k` * 2^`n` + 1 (a Proth prime),
-/// where `n` is relatively larger (e.g., greater than 32).
+/// where `n` is relatively large (e.g., greater than 32).
 pub trait StarkField: FieldElement<BaseField = Self> {
-    /// Type describing quadratic extension of this StarkField.
-    type QuadExtension: FieldElement<BaseField = Self>;
-
     /// Prime modulus of the field. Must be of the form `k` * 2^`n` + 1 (a Proth prime).
     /// This ensures that the field has high 2-adicity.
     const MODULUS: Self::PositiveInteger;
@@ -229,6 +235,30 @@ pub trait StarkField: FieldElement<BaseField = Self> {
     /// Returns byte representation of the field modulus in little-endian byte order.
     fn get_modulus_le_bytes() -> Vec<u8>;
 
-    /// Returns a canonical integer representation of the field element.
+    /// Returns a canonical integer representation of this field element.
     fn as_int(&self) -> Self::PositiveInteger;
 }
+
+// EXTENSIBLE FIELD
+// ================================================================================================
+
+/// Defines basic arithmetic in an extension of a StarkField of a given degree.
+///
+/// This trait defines how to perform multiplication and compute a Frobenius automorphisms of an
+/// element in an extension of degree N for a given [StarkField]. It as assumed that an element in
+/// degree N extension field can be represented by N field elements in the base field.
+///
+/// Implementation of this trait implicitly defines the irreducible polynomial over which the
+/// extension field is defined.
+pub trait ExtensibleField<const N: usize>: StarkField {
+    /// Returns a product of `a` and `b` in the field defined by this extension.
+    fn mul(a: [Self; N], b: [Self; N]) -> [Self; N];
+
+    /// Returns Frobenius automorphisms for `x` in the field defined by this extension.
+    fn frobenius(x: [Self; N]) -> [Self; N];
+
+    /// Returns true if this extension is supported for the underlying base field.
+    fn is_supported() -> bool {
+        true
+    }
+}
