@@ -27,14 +27,27 @@
 //!   very fast modular arithmetic including branchless multiplication and addition. To achieve
 //!   adequate security (i.e. ~100 bits), proofs must be generated in a quadratic extension of this
 //!   field. For higher levels of security, a cubic extension field should be used.
+//! * A 64-bit field with modulus 2<sup>64</sup> - 2<sup>32</sup> + 1. This field is about 15%
+//!   slower than the 62-bit field described above, but it has a number of other attractive
+//!   properties. To achieve adequate security (i.e. ~100 bits), proofs must be generated in a
+//!   quadratic extension of this field. For higher levels of security, a cubic extension field
+//!   should be used.
 //!
 //! ## Extension fields
 //!
-//! Currently, the library provides a generic way to create quadratic extensions of STARK fields.
-//! An extension element is defined as α + β * φ, where φ is a root of the polynomial
-//! x<sup>2</sup> - x - 1, and α and β are base field elements.
+//! Currently, the library provides a generic way to create quadratic and cubic extensions of
+//! supported STARK fields. This can be done by implementing [ExtensibleField] trait for
+//! degrees 2 and 3.
 //!
-//! Support for cubic extension fields is not yet available.
+//! Quadratic extension fields are defined using the following irreducible polynomials:
+//! * For [f62](crate::fields::f62) field, the polynomial is x<sup>2</sup> - x - 1.
+//! * For [f64](crate::fields::f64) field, the polynomial is x<sup>2</sup> - x + 2.
+//! * For [f128](crate::fields::f128) field, the polynomial is x<sup>2</sup> - x - 1.
+//!
+//! Cubic extension fields are defined using the following irreducible polynomials:
+//! * For [f62](crate::fields::f62) field, the polynomial is x<sup>3</sup> + 2x + 2.
+//! * For [f64](crate::fields::f64) field, the polynomial is x<sup>3</sup> - x - 1.
+//! * For [f128](crate::fields::f128) field, cubic extensions are not supported.
 //!
 //! # Polynomials
 //! [Polynomials](polynom) module implements basic polynomial operations such as:
@@ -84,7 +97,7 @@ pub mod fft;
 pub mod polynom;
 
 mod field;
-pub use field::{FieldElement, StarkField};
+pub use field::{ExtensibleField, FieldElement, StarkField};
 pub mod fields {
     //! Finite field implementations.
     //!
@@ -93,7 +106,9 @@ pub mod fields {
 
     pub use super::field::f128;
     pub use super::field::f62;
-    pub use super::field::QuadExtensionA;
+    pub use super::field::f64;
+    pub use super::field::CubeExtension;
+    pub use super::field::QuadExtension;
 }
 
 mod utils;
